@@ -50,9 +50,18 @@ static gboolean DOING_GDM_DEVELOPMENT = FALSE;
 static gboolean RUNNING_UNDER_GDM = FALSE;
 
 static gboolean gdm_running = FALSE;
+static int GdmMinimalUID = 100;
 
 static GladeXML *xml;
 
+static void
+setup_cursor (GdkCursorType type)
+{
+	GdkCursor *cursor = gdk_cursor_new (type);
+	gdk_window_set_cursor (GDK_ROOT_PARENT (), cursor);
+	gdk_cursor_destroy (cursor);
+}
+
 static void
 update_greeters (void)
 {
@@ -82,6 +91,11 @@ update_greeters (void)
 			g_free (ret);
 			goto check_update_error;
 		}
+
+		/* sanity */
+		if (pid <= 0)
+			continue;
+
 		if (kill (pid, SIGHUP) != 0)
 			have_error = TRUE;
 		p = strchr (p, ';');
@@ -362,8 +376,7 @@ setup_user_combo (const char *name, const char *key)
 	pwent = getpwent();
 	
 	while (pwent != NULL) {
-		/* FIXME: 100 is a pretty arbitrary constant */
-		if (pwent->pw_uid >= 100 &&
+		if (pwent->pw_uid >= GdmMinimalUID &&
 		    strcmp (ve_sure_string (str), pwent->pw_name) != 0) {
 			users = g_list_append (users,
 					       g_strdup (pwent->pw_name));
@@ -1002,8 +1015,14 @@ get_theme_dir (void)
 }
 
 static void
-selection_changed (GtkTreeSelection *selection,
-		   GtkWidget        *dialog)
+textview_set_buffer (GtkTextView *view, const char *text)
+{
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer (view);
+	gtk_text_buffer_set_text (buffer, text, -1);
+}
+
+static void
+selection_changed (GtkTreeSelection *selection, gpointer data)
 {
         GtkWidget *label;
         GtkWidget *preview = glade_helper_get (xml, "gg_theme_preview",
@@ -1021,27 +1040,27 @@ selection_changed (GtkTreeSelection *selection,
 	gtk_tree_model_get_value (model, &iter,
 				  THEME_COLUMN_AUTHOR,
 				  &value);
-        label = glade_helper_get (xml, "gg_author_label",
-				  GTK_TYPE_LABEL);
-	gtk_label_set_text (GTK_LABEL (label),
-			    ve_sure_string (g_value_get_string (&value)));
+        label = glade_helper_get (xml, "gg_author_text_view",
+				  GTK_TYPE_TEXT_VIEW);
+	textview_set_buffer (GTK_TEXT_VIEW (label),
+			     ve_sure_string (g_value_get_string (&value)));
 	g_value_unset (&value);
 
 	gtk_tree_model_get_value (model, &iter,
 				  THEME_COLUMN_COPYRIGHT,
 				  &value);
-        label = glade_helper_get (xml, "gg_copyright_label",
-				  GTK_TYPE_LABEL);
-	gtk_label_set_text (GTK_LABEL (label),
+        label = glade_helper_get (xml, "gg_copyright_text_view",
+				  GTK_TYPE_TEXT_VIEW);
+	textview_set_buffer (GTK_TEXT_VIEW (label),
 			    ve_sure_string (g_value_get_string (&value)));
 	g_value_unset (&value);
 
 	gtk_tree_model_get_value (model, &iter,
 				  THEME_COLUMN_DESCRIPTION,
 				  &value);
-        label = glade_helper_get (xml, "gg_desc_label",
-				  GTK_TYPE_LABEL);
-	gtk_label_set_text (GTK_LABEL (label),
+        label = glade_helper_get (xml, "gg_desc_text_view",
+				  GTK_TYPE_TEXT_VIEW);
+	textview_set_buffer (GTK_TEXT_VIEW (label),
 			    ve_sure_string (g_value_get_string (&value)));
 	g_value_unset (&value);
 
@@ -1095,7 +1114,12 @@ read_themes (GtkListStore *store, const char *theme_dir, DIR *dir,
 		if (dent->d_name[0] == '.')
 			continue;
 		n = g_strconcat (theme_dir, "/", dent->d_name,
-				 "/GdmGreeterTheme.info", NULL);
+				 "/GdmGreeterTheme.desktop", NULL);
+		if (access (n, R_OK) != 0) {
+			g_free (n);
+			n = g_strconcat (theme_dir, "/", dent->d_name,
+					 "/GdmGreeterTheme.info", NULL);
+		}
 		if (access (n, R_OK) != 0) {
 			g_free (n);
 			continue;
@@ -1333,10 +1357,19 @@ get_the_dir (FILE *fp, char **error)
 			return NULL;
 		}
 
-		s = g_strconcat (dir, "/GdmGreeterTheme.info", NULL);
-		if (strcmp (buf, s) == 0)
-			got_info = TRUE;
-		g_free (s);
+		if ( ! got_info) {
+			s = g_strconcat (dir, "/GdmGreeterTheme.info", NULL);
+			if (strcmp (buf, s) == 0)
+				got_info = TRUE;
+			g_free (s);
+		}
+
+		if ( ! got_info) {
+			s = g_strconcat (dir, "/GdmGreeterTheme.desktop", NULL);
+			if (strcmp (buf, s) == 0)
+				got_info = TRUE;
+			g_free (s);
+		}
 	}
 
 	if (got_info)
@@ -1886,11 +1919,17 @@ gdm_event (GSignalInvocationHint *ihint,
 	   const GValue	       *param_values,
 	   gpointer		data)
 {
+	GdkEvent *event;
+
 	/* HAAAAAAAAAAAAAAAAACK */
 	/* Since the user has not logged in yet and may have left/right
 	 * mouse buttons switched, we just translate every right mouse click
 	 * to a left mouse click */
-	GdkEvent *event = g_value_get_pointer ((GValue *)param_values);
+	if (n_param_values != 2 ||
+	    !G_VALUE_HOLDS (&param_values[1], GDK_TYPE_EVENT))
+	  return FALSE;
+	
+	event = g_value_get_boxed (&param_values[1]);
 	if ((event->type == GDK_BUTTON_PRESS ||
 	     event->type == GDK_2BUTTON_PRESS ||
 	     event->type == GDK_3BUTTON_PRESS ||
@@ -1917,7 +1956,7 @@ main (int argc, char *argv[])
 			    LIBGNOMEUI_MODULE /* module_info */,
 			    argc, argv,
 			    /* *GNOME_PARAM_POPT_TABLE, options, */
-			    GNOME_PARAM_CREATE_DIRECTORIES, FALSE,
+			    GNOME_PARAM_CREATE_DIRECTORIES, ! RUNNING_UNDER_GDM,
 			    NULL);
 
 	glade_gnome_init();
@@ -1928,6 +1967,9 @@ main (int argc, char *argv[])
 		char *gtkrc;
 		guint sid;
 
+		/* Set busy cursor */
+		setup_cursor (GDK_WATCH);
+
 		/* If we are running under gdm parse the GDM gtkRC */
 		gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
 		gtkrc = gnome_config_get_string (GDM_KEY_GTKRC);
@@ -1936,6 +1978,9 @@ main (int argc, char *argv[])
 			gtk_rc_parse (gtkrc);
 		g_free (gtkrc);
 
+		/* evil, but oh well */
+		g_type_class_ref (GTK_TYPE_WIDGET);	
+
 		/* also setup third button to work as first to work in reverse
 		 * situations transparently */
 		sid = g_signal_lookup ("event",
@@ -1961,12 +2006,23 @@ main (int argc, char *argv[])
 						GTK_MESSAGE_ERROR,
 						GTK_BUTTONS_OK,
 						_("You must be the superuser (root) to configure GDM.\n"));
+		if (RUNNING_UNDER_GDM)
+			setup_cursor (GDK_LEFT_PTR);
 		gtk_dialog_run (GTK_DIALOG (fatal_error));
 		exit (EXIT_FAILURE);
 	}
 
+	/* XXX: the setup proggie using a greeter config var for it's
+	 * ui?  Say it ain't so.  Our config sections are SUCH A MESS */
+	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
+	GdmMinimalUID = gnome_config_get_int (GDM_KEY_MINIMALUID);
+	gnome_config_pop_prefix ();
+
 	setup_gui ();
 
+	if (RUNNING_UNDER_GDM)
+		setup_cursor (GDK_LEFT_PTR);
+
 	gtk_main ();
 
 	return 0;
