@@ -1,5 +1,7 @@
 #include "config.h"
 
+#include <libintl.h>
+#include <locale.h>
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
@@ -11,6 +13,7 @@
 
 #include "gdm.h"
 #include "gdmwm.h"
+#include "vicious.h"
 
 #include "greeter.h"
 #include "greeter_configuration.h"
@@ -18,40 +21,76 @@
 #include "greeter_geometry.h"
 #include "greeter_item_clock.h"
 #include "greeter_item_pam.h"
+#include "greeter_item_capslock.h"
+#include "greeter_item_timed.h"
 #include "greeter_events.h"
 #include "greeter_action_language.h"
 #include "greeter_session.h"
+#include "greeter_system.h"
 
 gboolean DOING_GDM_DEVELOPMENT = FALSE;
 
 GtkWidget *window;
 GtkWidget *canvas;
 
-char *GreeterConfTheme = NULL;
+char *GdmGraphicalTheme = NULL;
+char *GdmGraphicalThemeDir = NULL;
 char *GdmDefaultLocale = NULL;
+int GdmXineramaScreen = 0;
 gboolean GdmShowGnomeChooserSession = FALSE;
 gboolean GdmShowGnomeFailsafeSession = FALSE;
 gboolean GdmShowXtermFailsafeSession = FALSE;
 gboolean GdmShowLastSession = FALSE;
 gchar *GdmSessionDir = NULL;
 gchar *GdmLocaleFile = NULL;
+gchar *GdmHalt = NULL;
+gchar *GdmReboot = NULL;
+gchar *GdmSuspend = NULL;
+gchar *GdmConfigurator = NULL;
+gboolean GdmSystemMenu = TRUE;
+gboolean GdmConfigAvailable = TRUE;
+gboolean GdmTimedLoginEnable;
+gboolean GdmUse24Clock;
+gchar *GdmTimedLogin;
+gint GdmTimedLoginDelay;
 
-int greeter_use_circles_in_entry = FALSE;
+gboolean GdmUseCirclesInEntry = FALSE;
 
+static gboolean used_defaults = FALSE;
+gint greeter_current_delay = 0;
+
+extern gboolean session_dir_whacked_out;
 
 
 static void 
 greeter_parse_config (void)
 {
-    if (!g_file_test (GDM_CONFIG_FILE, G_FILE_TEST_EXISTS)) {
+    if (!g_file_test (GDM_CONFIG_FILE, G_FILE_TEST_EXISTS))
+      {
 	syslog (LOG_ERR, _("greeter_parse_config: No configuration file: %s. Using defaults."), GDM_CONFIG_FILE);
-    }
+	used_defaults = TRUE;
+      }
 
     gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
 
-    GreeterConfTheme = gnome_config_get_string (GREETER_KEY_THEME);
+    GdmGraphicalTheme = gnome_config_get_string (GDM_KEY_GRAPHICAL_THEME);
+    if (GdmGraphicalTheme == NULL ||
+	GdmGraphicalTheme[0] == '\0')
+      {
+	g_free (GdmGraphicalTheme);
+	GdmGraphicalTheme = g_strdup ("circles");
+      }
+
+    GdmGraphicalThemeDir = gnome_config_get_string (GDM_KEY_GRAPHICAL_THEME_DIR);
+    if (GdmGraphicalThemeDir == NULL ||
+	! g_file_test (GdmGraphicalThemeDir, G_FILE_TEST_IS_DIR))
+      {
+        g_free (GdmGraphicalThemeDir);
+        GdmGraphicalThemeDir = g_strdup (GREETERTHEMEDIR);
+      }
     GdmDefaultLocale = gnome_config_get_string (GDM_KEY_LOCALE);
-    greeter_use_circles_in_entry = gnome_config_get_bool (GREETER_KEY_ENTRY_CIRCLES);
+    GdmXineramaScreen = gnome_config_get_int (GDM_KEY_XINERAMASCREEN);
+    GdmUseCirclesInEntry = gnome_config_get_bool (GDM_KEY_ENTRY_CIRCLES);
 
     GdmShowXtermFailsafeSession = gnome_config_get_bool (GDM_KEY_SHOW_XTERM_FAILSAFE);
     GdmShowGnomeFailsafeSession = gnome_config_get_bool (GDM_KEY_SHOW_GNOME_FAILSAFE);
@@ -59,8 +98,57 @@ greeter_parse_config (void)
     GdmShowLastSession = gnome_config_get_bool (GDM_KEY_SHOW_LAST_SESSION);
     GdmSessionDir = gnome_config_get_string (GDM_KEY_SESSDIR);
     GdmLocaleFile = gnome_config_get_string (GDM_KEY_LOCFILE);
-    
+    GdmSystemMenu = gnome_config_get_bool (GDM_KEY_SYSMENU);
+    GdmConfigAvailable = gnome_config_get_bool (GDM_KEY_CONFIG_AVAILABLE);
+    GdmHalt = gnome_config_get_string (GDM_KEY_HALT);
+    GdmReboot = gnome_config_get_string (GDM_KEY_REBOOT);
+    GdmSuspend = gnome_config_get_string (GDM_KEY_SUSPEND);
+    GdmConfigurator = gnome_config_get_string (GDM_KEY_CONFIGURATOR);
+
+    GdmTimedLoginEnable = gnome_config_get_bool (GDM_KEY_TIMED_LOGIN_ENABLE);
+
+    /* Note: TimedLogin here is not gotten out of the config
+     * but from the daemon since it's been munged on by the daemon a bit
+     * already maybe */
+    if (GdmTimedLoginEnable)
+      {
+        GdmTimedLogin = g_strdup (g_getenv("GDM_TIMED_LOGIN_OK"));
+	if (ve_string_empty (GdmTimedLogin))
+	  {
+	    g_free (GdmTimedLogin);
+	    GdmTimedLogin = NULL;
+	  }
+
+	GdmTimedLoginDelay =
+		gnome_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
+	if (GdmTimedLoginDelay < 5)
+	  {
+	    syslog (LOG_WARNING,
+		    _("TimedLoginDelay was less then 5.  I'll just use 5."));
+	    GdmTimedLoginDelay = 5;
+	  }
+      }
+    else
+      {
+        GdmTimedLogin = NULL;
+	GdmTimedLoginDelay = 5;
+      }
+    greeter_current_delay = GdmTimedLoginDelay;
+
+    GdmUse24Clock = gnome_config_get_bool (GDM_KEY_USE_24_CLOCK);
+
     gnome_config_pop_prefix();
+
+    if (GdmXineramaScreen < 0)
+      GdmXineramaScreen = 0;
+}
+
+static void
+setup_cursor (GdkCursorType type)
+{
+	GdkCursor *cursor = gdk_cursor_new (type);
+	gdk_window_set_cursor (GDK_ROOT_PARENT (), cursor);
+	gdk_cursor_destroy (cursor);
 }
 
 static gboolean
@@ -71,9 +159,11 @@ greeter_ctrl_handler (GIOChannel *source,
     gchar buf[PIPE_SIZE];
     gsize len;
     GtkWidget *dlg;
+    char *tmp;
     char *session;
     GreeterItemInfo *conversation_info;
     GreeterItemInfo *entry_info;
+    static GnomeCanvasItem *disabled_cover = NULL;
     gchar *language;
 
     /* If this is not incoming i/o then return */
@@ -104,34 +194,44 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 	
 	greeter_item_pam_set_user (buf);
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
     case GDM_LOGIN:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	greeter_item_pam_prompt (buf, 32, TRUE, TRUE);
+	tmp = ve_locale_to_utf8 (buf);
+	greeter_item_pam_prompt (tmp, 32, TRUE, TRUE);
+	g_free (tmp);
 	break;
 
     case GDM_PROMPT:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	greeter_item_pam_prompt (buf, 128, TRUE, FALSE);
+	tmp = ve_locale_to_utf8 (buf);
+	greeter_item_pam_prompt (tmp, 128, TRUE, FALSE);
+	g_free (tmp);
 	break;
 
     case GDM_NOECHO:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	greeter_item_pam_prompt (buf, 128, FALSE, FALSE);
+	tmp = ve_locale_to_utf8 (buf);
+	greeter_item_pam_prompt (tmp, 128, FALSE, FALSE);
+	g_free (tmp);
 	break;
 
     case GDM_MSG:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
-	greeter_item_pam_message (buf);
-	g_print ("%c\n", STX);
+	tmp = ve_locale_to_utf8 (buf);
+	greeter_item_pam_message (tmp);
+	g_free (tmp);
+	printf ("%c\n", STX);
+	fflush (stdout);
 
 	break;
 
@@ -139,9 +239,12 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	greeter_item_pam_error (buf);
+	tmp = ve_locale_to_utf8 (buf);
+	greeter_item_pam_error (tmp);
+	g_free (tmp);
 	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_ERRDLG:
@@ -151,12 +254,14 @@ greeter_ctrl_handler (GIOChannel *source,
 	/* we should be now fine for focusing new windows */
 	gdm_wm_focus_new_windows (TRUE);
 
+	tmp = ve_locale_to_utf8 (buf);
 	dlg = gtk_message_dialog_new (NULL /* parent */,
 				      GTK_DIALOG_MODAL /* flags */,
 				      GTK_MESSAGE_ERROR,
 				      GTK_BUTTONS_OK,
 				      "%s",
-				      buf);
+				      tmp);
+	g_free (tmp);
 
 	gdm_wm_center_window (GTK_WINDOW (dlg));
 
@@ -165,23 +270,30 @@ greeter_ctrl_handler (GIOChannel *source,
 	gtk_widget_destroy (dlg);
 	gdm_wm_no_login_focus_pop ();
 	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_SESS:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 	buf[len-1] = '\0';
 
-	session = greeter_session_lookup (buf);
-	g_print ("%c%s\n", STX, session);
+	tmp = ve_locale_to_utf8 (buf);
+	session = greeter_session_lookup (tmp);
+	g_free (tmp);
+	tmp = ve_locale_from_utf8 (session);
+	printf ("%c%s\n", STX, tmp);
+	fflush (stdout);
 	g_free (session);
+	g_free (tmp);
 	break;
 
     case GDM_LANG:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 	buf[len-1] = '\0';
 	language = greeter_language_get_language (buf);
-	g_print ("%c%s\n", STX, language);
+	printf ("%c%s\n", STX, language);
+	fflush (stdout);
 	g_free (language);
 	break;
 
@@ -189,9 +301,10 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
 	if (greeter_save_session ())
-	  g_print ("%cY\n", STX);
+	  printf ("%cY\n", STX);
 	else
-	  g_print ("%c\n", STX);
+	  printf ("%c\n", STX);
+	fflush (stdout);
 	
 	break;
 
@@ -199,9 +312,10 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
 	if (greeter_language_get_save_language ())
-	    g_print ("%cY\n", STX);
+	    printf ("%cY\n", STX);
 	else
-	    g_print ("%c\n", STX);
+	    printf ("%c\n", STX);
+	fflush (stdout);
 
 	break;
 
@@ -215,21 +329,28 @@ greeter_ctrl_handler (GIOChannel *source,
 	conversation_info = greeter_lookup_id ("pam-conversation");
 	
 	if (conversation_info)
-	  g_object_set (G_OBJECT (conversation_info->item),
-			"text",	buf,
-			NULL);
+	  {
+	    tmp = ve_locale_to_utf8 (buf);
+	    g_object_set (G_OBJECT (conversation_info->item),
+			  "text", tmp,
+			  NULL);
+	    g_free (tmp);
+	  }
 
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_QUIT:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
-	/* TODO: Stuff. */
+	greeter_item_timed_stop ();
+	greeter_item_pam_leftover_messages ();
 
 	gdk_flush ();
 
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 
 	/* screw gtk_main_quit, we want to make sure we definately die */
 	_exit (EXIT_SUCCESS);
@@ -243,7 +364,9 @@ greeter_ctrl_handler (GIOChannel *source,
 		do {
 			g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 			buf[len-1] = '\0';
-			g_string_append (str, buf);
+			tmp = ve_locale_to_utf8 (buf);
+			g_string_append (str, tmp);
+			g_free (tmp);
 		} while (len == PIPE_SIZE-1);
 
 
@@ -251,7 +374,10 @@ greeter_ctrl_handler (GIOChannel *source,
 
 		g_string_free (str, TRUE);
 
-		g_print ("%c%s\n", STX, sess);
+		tmp = ve_locale_from_utf8 (sess);
+		printf ("%c%s\n", STX, tmp);
+		fflush (stdout);
+		g_free (tmp);
 
 		g_free (sess);
 	}
@@ -261,46 +387,64 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
 	if (greeter_save_gnome_session ())
-	    g_print ("%cY\n", STX);
+	    printf ("%cY\n", STX);
 	else
-	    g_print ("%c\n", STX);
+	    printf ("%c\n", STX);
+	fflush (stdout);
 
 	break;
 
     case GDM_STARTTIMER:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
-	/*
-	 * Timed Login: Start Timer Loop
-	 */
-
-	/* TODO */
+	greeter_item_timed_start ();
 	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_STOPTIMER:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
-	/*
-	 * Timed Login: Stop Timer Loop
-	 */
+	greeter_item_timed_stop ();
 
-	/* TODO */
-	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_DISABLE:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
-	/* TODO */
-	g_print ("%c\n", STX);
+	gtk_widget_set_sensitive (window, FALSE);
+
+	if (disabled_cover == NULL)
+	  {
+	    disabled_cover = gnome_canvas_item_new
+		    (gnome_canvas_root (GNOME_CANVAS (canvas)),
+		     GNOME_TYPE_CANVAS_RECT,
+		     "x1", 0.0,
+		     "y1", 0.0,
+		     "x2", (double)canvas->allocation.width,
+		     "y2", (double)canvas->allocation.height,
+		     "fill_color_rgba", (guint)0x00000088,
+		     NULL);
+	  }
+
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_ENABLE:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
-	/* TODO */
-	g_print ("%c\n", STX);
+	gtk_widget_set_sensitive (window, TRUE);
+
+	if (disabled_cover != NULL)
+	  {
+	    gtk_object_destroy (GTK_OBJECT (disabled_cover));
+	    disabled_cover = NULL;
+	  }
+
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     /* These are handled separately so ignore them here and send
@@ -308,8 +452,39 @@ greeter_ctrl_handler (GIOChannel *source,
     case GDM_NEEDPIC:
     case GDM_READPIC:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
+
+    case GDM_NOFOCUS:
+	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
+
+	gdm_wm_no_login_focus_push ();
+	
+	printf ("%c\n", STX);
+	fflush (stdout);
+	break;
+
+    case GDM_FOCUS:
+	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
+
+	gdm_wm_no_login_focus_pop ();
+	
+	printf ("%c\n", STX);
+	fflush (stdout);
+	break;
+
+    case GDM_SAVEDIE:
+	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
+
+	/* Set busy cursor */
+	setup_cursor (GDK_WATCH);
+
+	gdm_wm_save_wm_order ();
+
+	gdk_flush ();
+
+	_exit (EXIT_SUCCESS);
 	
     default:
 	break;
@@ -338,9 +513,13 @@ greeter_setup_items (void)
   greeter_item_clock_setup ();
   greeter_item_pam_setup ();
   greeter_item_capslock_setup (window);
+  greeter_item_timed_setup ();
   greeter_item_register_action_callback ("language_button",
 					 greeter_action_language,
 					 window);
+  greeter_item_register_action_callback ("disconnect_button",
+					 (ActionFunc)gtk_main_quit,
+					 NULL);
   greeter_item_system_setup ();
   greeter_item_session_setup ();
 }
@@ -370,6 +549,7 @@ verify_gdm_version (void)
     {
       GtkWidget *dialog;
     
+      gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
       dialog = gtk_message_dialog_new (NULL /* parent */,
@@ -384,6 +564,8 @@ verify_gdm_version (void)
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
+
+      setup_cursor (GDK_LEFT_PTR);
     
       gtk_dialog_run (GTK_DIALOG (dialog));
     
@@ -396,6 +578,7 @@ verify_gdm_version (void)
     {
       GtkWidget *dialog;
     
+      gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
       dialog = gtk_message_dialog_new (NULL /* parent */,
@@ -417,6 +600,8 @@ verify_gdm_version (void)
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
       
+      setup_cursor (GDK_LEFT_PTR);
+
       switch (gtk_dialog_run (GTK_DIALOG (dialog)))
 	{
 	case RESPONSE_REBOOT:
@@ -436,6 +621,7 @@ verify_gdm_version (void)
     {
       GtkWidget *dialog;
     
+      gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
       dialog = gtk_message_dialog_new (NULL /* parent */,
@@ -462,6 +648,8 @@ verify_gdm_version (void)
     
       gtk_dialog_set_default_response (GTK_DIALOG (dialog), RESPONSE_RESTART);
       
+      setup_cursor (GDK_LEFT_PTR);
+
       switch (gtk_dialog_run (GTK_DIALOG (dialog)))
 	{
 	case RESPONSE_RESTART:
@@ -523,6 +711,8 @@ greeter_query (const gchar *msg)
 				      GTK_BUTTONS_YES_NO,
 				      "%s",
 				      msg);
+	gtk_label_set_use_markup
+		(GTK_LABEL (GTK_MESSAGE_DIALOG (req)->label), TRUE);
 
 	g_signal_connect (G_OBJECT (req), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
@@ -568,7 +758,12 @@ greeter_abort (const gchar *format, ...)
 static void
 greeter_reread_config (int sig)
 {
-	/* FIXME: reparse config stuff here */
+  /* Set busy cursor */
+  setup_cursor (GDK_WATCH);
+  /* FIXME: actually reparse config stuff here, instead of
+   * just requesting a restart */
+  gdm_wm_save_wm_order ();
+  _exit (DISPLAY_RESTARTGREETER);
 }
 
 static void
@@ -578,12 +773,70 @@ greeter_done (int sig)
 }
 
 
-static void
-setup_cursor (GdkCursorType type)
+static char *
+get_theme_file (const char *in, char **theme_dir)
 {
-	GdkCursor *cursor = gdk_cursor_new (type);
-	gdk_window_set_cursor (GDK_ROOT_PARENT (), cursor);
-	gdk_cursor_destroy (cursor);
+  char *file, *dir, *info, *s, *key;
+
+  if (in == NULL)
+    in = "circles";
+
+  *theme_dir = NULL;
+
+  if (g_path_is_absolute (in))
+    {
+      dir = g_strdup (in);
+    }
+  else
+    {
+      dir = NULL;
+      if (DOING_GDM_DEVELOPMENT)
+        {
+          dir = g_build_filename ("themes", in, NULL);
+	  if (access (dir, F_OK) != 0)
+	    {
+	      g_free (dir);
+	      dir = NULL;
+	    }
+	}
+      if (dir == NULL)
+        dir = g_build_filename (GdmGraphicalThemeDir, in, NULL);
+    }
+
+  *theme_dir = dir;
+
+  info = g_build_filename (dir, "GdmGreeterTheme.desktop", NULL);
+  if (access (info, F_OK) != 0) {
+	  g_free (info);
+	  info = g_build_filename (dir, "GdmGreeterTheme.info", NULL);
+  }
+  if (access (info, F_OK) != 0)
+    {
+      /* just guess the name, we have no info about the theme at
+       * this point */
+      g_free (info);
+      file = g_strdup_printf ("%s/%s.xml", dir, in);
+      return file;
+    }
+
+  key = g_strconcat ("=", info, "=/", NULL);
+  g_free (info);
+  gnome_config_push_prefix (key);
+  g_free (key);
+
+  s = gnome_config_get_translated_string ("GdmGreeterTheme/Greeter");
+  if (s == NULL || s[0] == '\0')
+    {
+      g_free (s);
+      s = g_strdup_printf ("%s.xml", in);
+    }
+
+  file = g_build_filename (dir, s, NULL);
+  g_free (s);
+
+  gnome_config_pop_prefix ();
+
+  return file;
 }
 
 int
@@ -593,7 +846,6 @@ main (int argc, char *argv[])
   struct sigaction term;
   sigset_t mask;
   GIOChannel *ctrlch;
-  gint w, h;
   GError *error;
   GreeterItemInfo *root;
   char *theme_file;
@@ -605,10 +857,9 @@ main (int argc, char *argv[])
 
   openlog ("gdmgreeter", LOG_PID, LOG_DAEMON);
 
-  setlocale (LC_ALL, "");
-
-  bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-  textdomain (PACKAGE);
+  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  textdomain (GETTEXT_PACKAGE);
 
   greeter_parse_config ();
 
@@ -620,12 +871,13 @@ main (int argc, char *argv[])
     setlocale (LC_ALL, GdmDefaultLocale);
   else
     setlocale (LC_ALL, "");
-  
+
   gtk_init (&argc, &argv);
 
-  setup_cursor (GDK_LEFT_PTR);
+  /* Should be a watch already, but anyway */
+  setup_cursor (GDK_WATCH);
   
-  gdm_wm_screen_init (0);
+  gdm_wm_screen_init (GdmXineramaScreen);
   
   r = verify_gdm_version ();
   if (r != 0)
@@ -665,6 +917,8 @@ main (int argc, char *argv[])
   
   if (! DOING_GDM_DEVELOPMENT) {
     ctrlch = g_io_channel_unix_new (STDIN_FILENO);
+    g_io_channel_set_encoding (ctrlch, NULL, NULL);
+    g_io_channel_set_buffered (ctrlch, FALSE);
     g_io_channel_init (ctrlch);
     g_io_add_watch (ctrlch, 
 		    G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
@@ -673,9 +927,6 @@ main (int argc, char *argv[])
     g_io_channel_unref (ctrlch);
   }
 
-  w = gdk_screen_width ();
-  h = gdk_screen_height ();
-
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
   if (DOING_GDM_DEVELOPMENT)
@@ -685,49 +936,179 @@ main (int argc, char *argv[])
   canvas = gnome_canvas_new_aa ();
   gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas),
 				  0.0, 0.0,
-				  (double) w,
-				  (double) h);
+				  (double) gdm_wm_screen.width,
+				  (double) gdm_wm_screen.height);
   gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
-  gtk_window_set_default_size (GTK_WINDOW (window), w, h);
+  gtk_window_set_default_size (GTK_WINDOW (window), 
+			       gdm_wm_screen.width, 
+			       gdm_wm_screen.height);
   gtk_container_add (GTK_CONTAINER (window), canvas);
 
-  if (g_path_is_absolute (GreeterConfTheme))
-    {
-      theme_file = g_strdup (GreeterConfTheme);
-    }
-  else
-    {
-      theme_file = NULL;
-      if (DOING_GDM_DEVELOPMENT)
-        {
-          theme_file = g_build_filename ("themes",
-					 GreeterConfTheme,
-					 NULL);
-	  if (access (theme_file, F_OK) != 0)
-	    {
-	      g_free (theme_file);
-	      theme_file = NULL;
-	    }
-	}
-      if (theme_file == NULL)
-        theme_file = g_build_filename (GREETERTHEMEDIR,
-				       GreeterConfTheme,
-				       NULL);
-    }
-
-  theme_dir = g_path_get_dirname (theme_file);
+  theme_file = get_theme_file (GdmGraphicalTheme, &theme_dir);
   
   error = NULL;
   root = greeter_parse (theme_file, theme_dir,
-			GNOME_CANVAS (canvas), w, h, &error);
+			GNOME_CANVAS (canvas), 
+			gdm_wm_screen.width,
+			gdm_wm_screen.height,
+			&error);
+
+  /* Try circles.xml */
+  if (root == NULL)
+    {
+      g_free (theme_file);
+      g_free (theme_dir);
+      theme_file = get_theme_file ("circles", &theme_dir);
+      root = greeter_parse (theme_file, theme_dir,
+			    GNOME_CANVAS (canvas), 
+			    gdm_wm_screen.width,
+			    gdm_wm_screen.height,
+			    NULL);
+    }
 
   if (root == NULL)
-    g_warning ("Failed to parse file: %s!", error->message);
+    {
+      GtkWidget *dialog;
+
+      gdm_wm_init (0);
+      gdm_wm_focus_new_windows (TRUE);
+    
+      dialog = gtk_message_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("There was an error loading the "
+					 "theme, and the default theme "
+					 "also could not have been loaded, "
+					 "I will attempt to start the "
+					 "standard greeter"));
+    
+      gtk_widget_show_all (dialog);
+      gdm_wm_center_window (GTK_WINDOW (dialog));
+
+      setup_cursor (GDK_LEFT_PTR);
+    
+      gtk_dialog_run (GTK_DIALOG (dialog));
+
+      execl (EXPANDED_BINDIR "/gdmlogin", EXPANDED_BINDIR "/gdmlogin", NULL);
+      execlp ("gdmlogin", "gdmlogin", NULL);
+
+      dialog = gtk_message_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("I could not start the standard "
+					 "greeter.  This display will abort "
+					 "and you may have to login another "
+					 "way and fix the installation of "
+					 "gdm"));
+    
+      gtk_widget_show_all (dialog);
+      gdm_wm_center_window (GTK_WINDOW (dialog));
+
+      setup_cursor (GDK_LEFT_PTR);
+    
+      gtk_dialog_run (GTK_DIALOG (dialog));
+
+      _exit (DISPLAY_ABORT);
+    }
 
   greeter_layout (root, GNOME_CANVAS (canvas));
   
   greeter_setup_items ();
+
   gtk_widget_show_all (window);
+  gtk_window_move (GTK_WINDOW (window), gdm_wm_screen.x, gdm_wm_screen.y);
+  gtk_widget_show_now (window);
+
+  /* can it ever happen that it'd be NULL here ??? */
+  if (window->window != NULL)
+    {
+      gdm_wm_init (GDK_WINDOW_XWINDOW (window->window));
+
+      /* Run the focus, note that this will work no matter what
+       * since gdm_wm_init will set the display to the gdk one
+       * if it fails */
+      gdm_wm_focus_window (GDK_WINDOW_XWINDOW (window->window));
+    }
+
+  if (session_dir_whacked_out)
+    {
+      GtkWidget *dialog;
+
+      gdm_wm_focus_new_windows (TRUE);
+
+      dialog = gtk_message_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("Your session directory is missing or empty!\n\n"
+					 "There are two available sessions you can use, but\n"
+					 "you should log in and correct the gdm configuration."));
+      gtk_widget_show_all (dialog);
+      gdm_wm_center_window (GTK_WINDOW (dialog));
+
+      setup_cursor (GDK_LEFT_PTR);
+
+      gdm_wm_no_login_focus_push ();
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      gdm_wm_no_login_focus_pop ();
+    }
+
+  if (g_getenv ("GDM_WHACKED_GREETER_CONFIG") != NULL)
+    {
+      GtkWidget *dialog;
+
+      gdm_wm_focus_new_windows (TRUE);
+
+      dialog = gtk_message_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("The configuration file contains an invalid command\n"
+					 "line for the login dialog, and thus I ran the\n"
+					 "default command.  Please fix your configuration."));
+      gtk_widget_show_all (dialog);
+      gdm_wm_center_window (GTK_WINDOW (dialog));
+
+      setup_cursor (GDK_LEFT_PTR);
+
+      gdm_wm_no_login_focus_push ();
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      gdm_wm_no_login_focus_pop ();
+  }
+
+  /* There was no config file */
+  if (used_defaults)
+    {
+      GtkWidget *dialog;
+
+      gdm_wm_focus_new_windows (TRUE);
+
+      dialog = gtk_message_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("The configuration was not found.  GDM is using\n"
+					 "defaults to run this session.  You should log in\n"
+					 "and create a configuration file with the GDM\n"
+					 "configuration program."));
+      gtk_widget_show_all (dialog);
+      gdm_wm_center_window (GTK_WINDOW (dialog));
+
+      setup_cursor (GDK_LEFT_PTR);
+
+      gdm_wm_no_login_focus_push ();
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      gdm_wm_no_login_focus_pop ();
+    }
+
+  gdm_wm_restore_wm_order ();
+
+  setup_cursor (GDK_LEFT_PTR);
 
   gtk_main ();
 
